@@ -1,24 +1,10 @@
 /* ============================================================
 *
-* Copyright (C) 2007-2012 by Kåre Särs <kare.sars@iki .fi>
-* Copyright (C) 2009 by Arseniy Lartsev <receive-spam at yandex dot ru>
-* Copyright (C) 2014 by Gregor Mitsch: port to KDE5 frameworks
+* SPDX-FileCopyrightText: 2007-2012 Kåre Särs <kare.sars@iki .fi>
+* SPDX-FileCopyrightText: 2009 Arseniy Lartsev <receive-spam at yandex dot ru>
+* SPDX-FileCopyrightText: 2014 Gregor Mitsch : port to KDE5 frameworks
 *
-* This program is free software; you can redistribute it and/or
-* modify it under the terms of the GNU General Public License as
-* published by the Free Software Foundation; either version 2 of
-* the License or (at your option) version 3 or any later version
-* accepted by the membership of KDE e.V. (or its successor approved
-*  by the membership of KDE e.V.), which shall act as a proxy
-* defined in Section 14 of version 3 of the license.
-*
-* This program is distributed in the hope that it will be useful,
-* but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-* GNU General Public License for more details.
-*
-* You should have received a copy of the GNU General Public License.
-* along with this program.  If not, see <http://www.gnu.org/licenses/>
+* SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
 *
 * ============================================================ */
 
@@ -68,9 +54,10 @@ Skanlite::Skanlite(const QString &device, QWidget *parent)
 
     QDialogButtonBox *dlgButtonBoxBottom = new QDialogButtonBox(this);
     dlgButtonBoxBottom->setStandardButtons(QDialogButtonBox::Help | QDialogButtonBox::Close);
-    // was "User2:
-    QPushButton *btnAbout = dlgButtonBoxBottom->addButton(i18n("About"), QDialogButtonBox::ButtonRole::ActionRole);
-    // was "User1":
+    QPushButton *btnAbout = dlgButtonBoxBottom->addButton(i18n("About"), QDialogButtonBox::ButtonRole::HelpRole);
+    btnAbout->setIcon(QIcon::fromTheme(QStringLiteral("skanlite")));
+    QPushButton *btnReselectDevice = dlgButtonBoxBottom->addButton(i18n("Reselect scanner device"), QDialogButtonBox::ButtonRole::ActionRole);
+    btnReselectDevice->setIcon(QIcon::fromTheme(QStringLiteral("view-refresh")));
     QPushButton *btnSettings = dlgButtonBoxBottom->addButton(i18n("Settings"), QDialogButtonBox::ButtonRole::ActionRole);
     btnSettings->setIcon(QIcon::fromTheme(QStringLiteral("configure")));
 
@@ -78,9 +65,9 @@ Skanlite::Skanlite(const QString &device, QWidget *parent)
 
     m_ksanew = new KSaneIface::KSaneWidget(this);
     connect(m_ksanew, &KSaneWidget::scannedImageReady, this, &Skanlite::imageReady);
-    connect(m_ksanew, &KSaneWidget::availableDevices, this, &Skanlite::availableDevices);
     connect(m_ksanew, &KSaneWidget::userMessage, this, &Skanlite::alertUser);
     connect(m_ksanew, &KSaneWidget::buttonPressed, this, &Skanlite::buttonPressed);
+    connect(m_ksanew, &KSaneWidget::openedDeviceInfoUpdated, this, &Skanlite::updateWindowTitle);
     connect(m_ksanew, &KSaneWidget::scanDone, this, [this](){
         if (!m_pendingApplyScanOpts.isEmpty()) {
             applyScannerOptions(m_pendingApplyScanOpts);
@@ -107,10 +94,21 @@ Skanlite::Skanlite(const QString &device, QWidget *parent)
     QSize rect = window.readEntry("Geometry", QSize(740, 400));
     resize(rect);
 
+    // open scanner device from command line, otherwise try remembered one
+    QString deviceName;
+    if (device.isEmpty()) {
+        KConfigGroup general(KSharedConfig::openConfig(), QStringLiteral("General"));
+        deviceName = general.readEntry(QStringLiteral("deviceName"));
+    } else {
+        deviceName = device;
+    }
+
     connect(dlgButtonBoxBottom, &QDialogButtonBox::rejected, this, &QDialog::close);
     connect(this, &QDialog::finished, this, &Skanlite::saveWindowSize);
+    connect(this, &QDialog::finished, this, &Skanlite::saveScannerDevice);
     connect(this, &QDialog::finished, this, &Skanlite::saveScannerOptions);
     connect(btnSettings, &QPushButton::clicked, this, &Skanlite::showSettingsDialog);
+    connect(btnReselectDevice, &QPushButton::clicked, this, &Skanlite::reselectScannerDevice);
     connect(btnAbout, &QPushButton::clicked, this, &Skanlite::showAboutDialog);
     connect(dlgButtonBoxBottom, &QDialogButtonBox::helpRequested, this, &Skanlite::showHelp);
 
@@ -146,29 +144,24 @@ Skanlite::Skanlite(const QString &device, QWidget *parent)
         m_filterList.insert(0, QStringLiteral("image/png"));
         m_filterList.insert(1, QStringLiteral("image/jpeg"));
         m_filterList.insert(2, QStringLiteral("image/tiff"));
+        m_filterList.insert(3, QStringLiteral("application/pdf"));
 
         m_filter16BitList << QStringLiteral("image/png");
         m_filter16BitList << QStringLiteral("image/tiff");
 
-        // fill m_filterList (...) and m_typeList (list of file suffixes)
+        // fill m_filterList (...)
         {
             QStringList namedMimeTypes;
             for (const QString &mimeStr : qAsConst(m_filterList)) {
                 QMimeType mimeType = QMimeDatabase().mimeTypeForName(mimeStr);
                 namedMimeTypes.append(mimeType.name());
 
-                QStringList fileSuffixes = mimeType.suffixes();
-
-                if (fileSuffixes.size() > 0) {
-                    m_typeList << fileSuffixes.first();
-                }
+                m_settingsUi.imgFormat->addItem(mimeType.preferredSuffix(), mimeType.name());
+                m_saveLocation->addImageFormat(mimeType.preferredSuffix(), mimeType.name());
             }
             m_filterList << std::move(namedMimeTypes);
         }
 
-        m_settingsUi.imgFormat->addItems(m_typeList);
-        m_saveLocation->setImageFormats(m_typeList);
-
         mainLayout->addWidget(settingsWidget);
 
         QDialogButtonBox *dlgButtonBoxBottom = new QDialogButtonBox(this);
@@ -186,11 +179,11 @@ Skanlite::Skanlite(const QString &device, QWidget *parent)
         // default directory for the save dialog
         m_saveLocation->setFolderUrl(m_settingsUi.saveDirRequester->url());
         m_saveLocation->setImagePrefix(m_settingsUi.imgPrefix->text());
-        m_saveLocation->setImageFormat(m_settingsUi.imgFormat->currentText());
+        m_saveLocation->setImageFormatIndex(m_settingsUi.imgFormat->currentIndex());
     }
 
     // open the scan device
-    if (m_ksanew->openDevice(device) == false) {
+    if (m_ksanew->openDevice(deviceName) == false) {
         QString dev = m_ksanew->selectDevice(nullptr);
         if (dev.isEmpty()) {
             // either no scanner was found or then cancel was pressed.
@@ -202,13 +195,13 @@ Skanlite::Skanlite(const QString &device, QWidget *parent)
             exit(1);
         }
         else {
-            setWindowTitle(i18nc("@title:window %1 = scanner maker, %2 = scanner model", "%1 %2 - Skanlite", m_ksanew->deviceVendor(), m_ksanew->deviceModel()));
-            m_deviceName = QStringLiteral("%1:%2").arg(m_ksanew->deviceVendor(), m_ksanew->deviceModel());
+            updateWindowTitle(dev, m_ksanew->deviceVendor(), m_ksanew->deviceModel());
+            m_deviceName = dev;
         }
     }
     else {
-        setWindowTitle(i18nc("@title:window %1 = scanner device", "%1 - Skanlite", device));
-        m_deviceName = device;
+        updateWindowTitle(deviceName);
+        m_deviceName = deviceName;
     }
 
     // prepare the Show Image Dialog
@@ -223,6 +216,7 @@ Skanlite::Skanlite(const QString &device, QWidget *parent)
     loadScannerOptions();
 
     m_firstImage = true;
+    m_ksanew->setFocus();
 
     if (m_dbusInterface.setupDBusInterface()) {
         // D-Bus related slots
@@ -259,6 +253,7 @@ void Skanlite::showHelp()
 void Skanlite::closeEvent(QCloseEvent *event)
 {
     saveWindowSize();
+    saveScannerDevice();
     saveScannerOptions();
     event->accept();
 }
@@ -270,6 +265,28 @@ void Skanlite::saveWindowSize()
     window.sync();
 }
 
+void Skanlite::saveScannerDevice()
+{
+    KConfigGroup general(KSharedConfig::openConfig(), "General");
+    general.writeEntry("deviceName", m_ksanew->deviceName());
+    general.sync();
+}
+
+void Skanlite::reselectScannerDevice()
+{
+    m_ksanew->closeDevice();
+    // open the scan device dialog
+    QString dev = m_ksanew->selectDevice(nullptr);
+    if (m_ksanew->openDevice(dev) == false) {
+        // could not open a scanner
+        KMessageBox::sorry(nullptr, i18n("Opening the selected scanner failed."));
+    }
+    else {
+        updateWindowTitle(dev, m_ksanew->deviceVendor(), m_ksanew->deviceModel());
+        m_deviceName = dev;
+    }
+}
+
 // Pops up message box similar to what perror() would print
 //************************************************************
 static void perrorMessageBox(const QString &text)
@@ -296,7 +313,12 @@ void Skanlite::readSettings(void)
     }
     m_settingsUi.saveDirRequester->setUrl(saving.readEntry("Location", QUrl(QDir::homePath())));
     m_settingsUi.imgPrefix->setText(saving.readEntry("NamePrefix", i18nc("prefix for auto naming", "Image-")));
-    m_settingsUi.imgFormat->setCurrentText(saving.readEntry("ImgFormat", "png"));
+    QString format = saving.readEntry("ImgFormat", "image/png");
+    int index = m_settingsUi.imgFormat->findData(format);
+    if (index >= 0) {
+        m_settingsUi.imgFormat->setCurrentIndex(index);
+    }
+
     m_settingsUi.imgQuality->setValue(saving.readEntry("ImgQuality", 90));
     m_settingsUi.setQuality->setChecked(saving.readEntry("SetQuality", false));
     m_settingsUi.showB4Save->setChecked(saving.readEntry("ShowBeforeSave", true));
@@ -328,7 +350,7 @@ void Skanlite::showSettingsDialog(void)
         saving.writeEntry("SaveMode", m_settingsUi.saveModeCB->currentIndex());
         saving.writeEntry("Location", m_settingsUi.saveDirRequester->url());
         saving.writeEntry("NamePrefix", m_settingsUi.imgPrefix->text());
-        saving.writeEntry("ImgFormat", m_settingsUi.imgFormat->currentText());
+        saving.writeEntry("ImgFormat", m_settingsUi.imgFormat->currentData().toString());
         saving.writeEntry("SetQuality", m_settingsUi.setQuality->isChecked());
         saving.writeEntry("ImgQuality", m_settingsUi.imgQuality->value());
         saving.writeEntry("ShowBeforeSave", m_settingsUi.showB4Save->isChecked());
@@ -353,7 +375,7 @@ void Skanlite::showSettingsDialog(void)
         // pressing OK in the settings dialog means use those settings.
         m_saveLocation->setFolderUrl(m_settingsUi.saveDirRequester->url());
         m_saveLocation->setImagePrefix(m_settingsUi.imgPrefix->text());
-        m_saveLocation->setImageFormat(m_settingsUi.imgFormat->currentText());
+        m_saveLocation->setImageFormatIndex(m_settingsUi.imgFormat->currentIndex());
 
         m_firstImage = true;
     }
@@ -387,11 +409,7 @@ bool urlExists(const QUrl& url)
         }
     }
     else {
-#if KIO_VERSION < QT_VERSION_CHECK(5, 69, 0)
-        KIO::StatJob *statJob = KIO::stat(url, KIO::StatJob::DestinationSide, 0);
-#else
         KIO::StatJob *statJob = KIO::statDetails(url, KIO::StatJob::DestinationSide, KIO::StatNoDetails);
-#endif
         KJobWidgets::setWindow(statJob, QApplication::activeWindow());
         if (!statJob->exec()) {
             return false;
@@ -424,21 +442,21 @@ void Skanlite::saveImage()
         m_saveLocation->setFolderUrl(dirUrl);
     }
 
-
     QString prefix = m_saveLocation->imagePrefix();
-    QString imgFormat = m_saveLocation->imageFormat().toLower();
+    QString imageMimetype = m_saveLocation->imageMimetype();
     int fileNumber = m_saveLocation->startNumber();
-    QStringList filterList = m_filterList;
+    QStringList filterList;
 
     if ((m_img.format() == QImage::Format_Grayscale16) ||
         (m_img.format() == QImage::Format_RGBX64))
     {
         filterList = m_filter16BitList;
-        if (imgFormat != QLatin1String("png") && imgFormat != QLatin1String("tif")
-            && imgFormat != QLatin1String("tiff")) {
-            imgFormat = QStringLiteral("png");
+        if (imageMimetype != QLatin1String("image/png") && imageMimetype != QLatin1String("image/tiff")) {
+            imageMimetype = QStringLiteral("image/png");
             KMessageBox::information(this, i18n("The image will be saved in the PNG format, as the selected image type does not support saving 16 bit color images."));
         }
+    } else {
+        filterList = m_filterList;
     }
 
     // find next available file name for name suggestion
@@ -448,7 +466,7 @@ void Skanlite::saveImage()
         fname = QStringLiteral("%1%2.%3")
                 .arg(prefix)
                 .arg(i, 4, 10, QLatin1Char('0'))
-                .arg(imgFormat);
+                .arg(m_saveLocation->imageSuffix());
 
         fileUrl = dirUrl;
         fileUrl.setPath(fileUrl.path() + fname);
@@ -470,10 +488,8 @@ void Skanlite::saveImage()
         // NOTE it is probably a bug that both setDirectoryUrl and selectUrl have
         // to be set to get remote urls to work
 
-        QStringList actualFilterList = filterList;
-        QString currentMimeFilter = QLatin1String("image/") + imgFormat;
-        saveDialog.setMimeTypeFilters(actualFilterList);
-        saveDialog.selectMimeTypeFilter(currentMimeFilter);
+        saveDialog.setMimeTypeFilters(filterList);
+        saveDialog.selectMimeTypeFilter(imageMimetype);
 
         if (saveDialog.exec() != QFileDialog::Accepted) {
             return;
@@ -496,6 +512,9 @@ void Skanlite::saveImage()
     if (suffix.isEmpty()) {
         fileFormat = QStringLiteral("png");
     }
+    if (suffix == QLatin1String("pdf")) {
+        fileFormat = QStringLiteral("pdf");
+    }
 
     if (!fileUrl.isLocalFile()) {
         QTemporaryFile tmp;
@@ -515,7 +534,7 @@ void Skanlite::saveImage()
     SkanliteImageSaver *imageSaver = new SkanliteImageSaver(this);
     connect(imageSaver, &SkanliteImageSaver::imageSaved, this, &Skanlite::imageSaved);
 
-    imageSaver->saveQImage(fileUrl, localName, m_img, static_cast<int>(m_ksanew->currentDPI()), fileFormat, quality);
+    imageSaver->saveQImage(fileUrl, localName, m_img, fileFormat, quality);
 
     m_showImgDialog->blockSignals(true);
     m_showImgDialog->close(); // calling close() on a closed window does nothing.
@@ -585,6 +604,7 @@ void Skanlite::imageSaved(const QUrl &fileUrl, const QString &localName, bool su
         Q_EMIT m_dbusInterface.imageSaved(localName);
     }
     m_ksanew->setDisabled(false);
+    m_ksanew->setFocus();
     m_saveUpdateTimer.stop();
     m_saveProgressBar->setVisible(false);
 
@@ -625,10 +645,12 @@ void Skanlite::saveScannerOptions()
         return;
     }
 
-    KConfigGroup options(KSharedConfig::openConfig(), QStringLiteral("Options For %1").arg(m_deviceName));
-    QMap <QString, QString> opts;
-    m_ksanew->getOptVals(opts);
-    writeScannerOptions(QStringLiteral("Options For %1").arg(m_deviceName), opts);
+    if (!m_deviceName.isEmpty()) {
+        KConfigGroup options(KSharedConfig::openConfig(), QStringLiteral("Options For %1").arg(m_deviceName));
+        QMap <QString, QString> opts;
+        m_ksanew->getOptVals(opts);
+        writeScannerOptions(QStringLiteral("Options For %1").arg(m_deviceName), opts);
+    }
 }
 
 void Skanlite::defaultScannerOptions()
@@ -651,22 +673,17 @@ void Skanlite::applyScannerOptions(const QMap <QString, QString> &opts)
 
 void Skanlite::loadScannerOptions()
 {
-    KConfigGroup saving(KSharedConfig::openConfig(), "Image Saving");
-    m_saveLocation->setStartNumber(saving.readEntry("NumberStartsFrom", 1));
+    if (!m_deviceName.isEmpty()) {
+        KConfigGroup saving(KSharedConfig::openConfig(), "Image Saving");
+        m_saveLocation->setStartNumber(saving.readEntry("NumberStartsFrom", 1));
 
-    if (!m_ksanew) {
-        return;
-    }
+        if (!m_ksanew) {
+            return;
+        }
 
-    QMap <QString, QString> opts;
-    readScannerOptions(QStringLiteral("Options For %1").arg(m_deviceName), opts);
-    applyScannerOptions(opts);
-}
-
-void Skanlite::availableDevices(const QList<KSaneWidget::DeviceInfo> &deviceList)
-{
-    for (const KSaneWidget::DeviceInfo &device : deviceList) {
-        qCDebug(SKANLITE_LOG) << device.name;
+        QMap <QString, QString> opts;
+        readScannerOptions(QStringLiteral("Options For %1").arg(m_deviceName), opts);
+        applyScannerOptions(opts);
     }
 }
 
@@ -739,21 +756,36 @@ void Skanlite::processSelectionOptions(QMap<QString, QString> &opts, bool ignore
     }
 }
 
+void Skanlite::updateWindowTitle(const QString &deviceName, const QString &deviceVendor, const QString &deviceModel)
+{
+    if (!deviceVendor.isEmpty() &&  !deviceModel.isEmpty()) {
+        setWindowTitle(i18nc("@title:window %1 = scanner maker, %2 = scanner model", "%1 %2 - Skanlite", deviceVendor, deviceModel));
+    } else if (!deviceName.isEmpty()) {
+        setWindowTitle(i18nc("@title:window %1 = scanner device", "%1 - Skanlite", deviceName));
+    } else {
+        setWindowTitle(i18n("Skanlite"));
+    }
+}
+
 // D-Bus interface related slots
 
 void Skanlite::getScannerOptions()
 {
-    QMap <QString, QString> opts;
-    m_ksanew->getOptVals(opts);
-    m_dbusInterface.setReply(serializeScannerOptions(opts));
+    if (!m_deviceName.isEmpty()) {
+        QMap <QString, QString> opts;
+        m_ksanew->getOptVals(opts);
+        m_dbusInterface.setReply(serializeScannerOptions(opts));
+    }
 }
 
 void Skanlite::setScannerOptions(const QStringList &options, bool ignoreSelection)
 {
-    QMap <QString, QString> opts;
-    deserializeScannerOptions(options, opts);
-    processSelectionOptions(opts, ignoreSelection);
-    applyScannerOptions(opts);
+    if (!m_deviceName.isEmpty()) {
+        QMap <QString, QString> opts;
+        deserializeScannerOptions(options, opts);
+        processSelectionOptions(opts, ignoreSelection);
+        applyScannerOptions(opts);
+    }
 }
 
 
@@ -766,42 +798,50 @@ static const QLatin1String defaultProfileGroup("Options For %1 - Profile %2"); /
 
 void Skanlite::saveScannerOptionsToProfile(const QStringList &options, const QString &profile, bool ignoreSelection)
 {
-    QMap <QString, QString> opts;
-    deserializeScannerOptions(options, opts);
-    processSelectionOptions(opts, ignoreSelection);
-    writeScannerOptions(QString(defaultProfileGroup).arg(m_deviceName, profile), opts);
+    if (!m_deviceName.isEmpty()) {
+        QMap <QString, QString> opts;
+        deserializeScannerOptions(options, opts);
+        processSelectionOptions(opts, ignoreSelection);
+        writeScannerOptions(QString(defaultProfileGroup).arg(m_deviceName, profile), opts);
+    }
 }
 
 void Skanlite::switchToProfile(const QString &profile, bool ignoreSelection)
 {
-    QMap <QString, QString> opts;
-    readScannerOptions(QString(defaultProfileGroup).arg(m_deviceName, profile), opts);
+    if (!m_deviceName.isEmpty()) {
+        QMap <QString, QString> opts;
+        readScannerOptions(QString(defaultProfileGroup).arg(m_deviceName, profile), opts);
 
-    if (opts.empty()) {
-        opts = m_defaultScanOpts;
+        if (opts.empty()) {
+            opts = m_defaultScanOpts;
+        }
+
+        processSelectionOptions(opts, ignoreSelection);
+        applyScannerOptions(opts);
     }
-
-    processSelectionOptions(opts, ignoreSelection);
-    applyScannerOptions(opts);
 }
 
 void Skanlite::getDeviceName()
 {
-    m_dbusInterface.setReply(QStringList(m_deviceName));
+    if (!m_deviceName.isEmpty()) {
+        m_dbusInterface.setReply(QStringList(m_deviceName));
+    }
 }
 
 void Skanlite::getSelection()
 {
-    QMap <QString, QString> opts;
-    m_ksanew->getOptVals(opts);
+    if (!m_deviceName.isEmpty()) {
+        QMap <QString, QString> opts;
+        m_ksanew->getOptVals(opts);
 
-    QStringList reply;
-    for (const auto &key : selectionSettings ) {
-        if (opts.contains(key)) {
-            reply.append(key + QLatin1Char('=') + opts[key]);
+        QStringList reply;
+        for (const auto &key : selectionSettings ) {
+            if (opts.contains(key)) {
+                reply.append(key + QLatin1Char('=') + opts[key]);
+            }
         }
+        m_dbusInterface.setReply(reply);
     }
-    m_dbusInterface.setReply(reply);
 }
 
 void Skanlite::setSelection(const QStringList &options)
